package com.cultural.heritage.order;

import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.exception.BusinessException;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class OrderIdGenerator {

    // 定义日期格式化器，精确到秒
    private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyyMMddHHmmss");

    // 定义序列号最大值
    private static final int MAX_SEQUENCE = 999999;

    // 当前日期，初始化为当天
    private static String currentDate = getCurrentDate();

    // 自增序列号，线程安全
    private static final AtomicInteger sequence = new AtomicInteger(0);

    // 锁对象，用于保护自增序列号的生成
    private static final Lock lock = new ReentrantLock();


    public static String generateOrderId() {
        // 获取当前日期
        String today = getCurrentDate();

        // 如果日期发生变化，重置序列号
        if (!today.equals(currentDate)) {
            currentDate = today;
            sequence.set(0);  // 重置序列号
        }

        // 获取时间戳（精确到秒）
        String timestamp = DATE_FORMAT.format(new Date());

        // 获取4位随机数
        String randomNumber = generateRandomNumber();

        // 获取6位自增序列号，并确保不超过最大值
        int seq = getNextSequence();

        // 格式化序列号为6位
        String formattedSequence = String.format("%06d", seq);

        // 拼接生成订单号
        return timestamp + randomNumber + formattedSequence;
    }

    // 获取当前日期（格式：yyyyMMdd）
    private static String getCurrentDate() {
        return new SimpleDateFormat("yyyyMMdd").format(new Date());
    }

    // 生成4位随机数（范围：0000到9999）
    private static String generateRandomNumber() {
        int random = (int) (Math.random() * 10000); // 生成0到9999之间的随机数
        return String.format("%04d", random); // 格式化为4位
    }

    // 获取下一个自增序列号，使用ReentrantLock来确保线程安全
    private static int getNextSequence() {
        lock.lock();  // 获取锁
        try {
            int seq = sequence.incrementAndGet();
            if (seq > MAX_SEQUENCE) {
                sequence.set(0);  // 达到最大值后重置
                seq = sequence.incrementAndGet();
            }
            return seq;
        } finally {
            lock.unlock();  // 确保在最终释放锁
        }
    }



    // 模拟高并发生成订单号
    public static void main(String[] args) throws InterruptedException {
        // 创建一个线程池
        ExecutorService executorService = Executors.newFixedThreadPool(100);
        ConcurrentHashMap<String, Integer> map = new ConcurrentHashMap<>();
        // 记录生成的订单号数量
        final int totalOrders = 999999;
        final CountDownLatch latch = new CountDownLatch(totalOrders);

        // 使用线程池并发生成订单号
        for (int i = 0; i < totalOrders; i++) {
            executorService.submit(() -> {
                // 每个线程生成一个订单号
                String orderId = generateOrderId();

                map.compute(orderId.substring(orderId.length() - 6), (key, value) -> {
                    if (value == null) {
                        return 1;
                    } else {
                        return value + 1;  // 如果有重复，则计数
                    }
                });

                System.out.println(orderId.substring(orderId.length() - 6));
                latch.countDown();  // 完成一个订单号生成
            });
        }


        // 等待所有订单号生成完成
        latch.await();

        // 关闭线程池
        executorService.shutdown();

        System.out.println("所有订单号生成完毕！");

        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            if (entry.getValue() > 1) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "订单号重复, 订单号为：" + entry.getKey());
            }
        }
        System.out.println("订单号唯一性验证通过！");
    }

}

